/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 */
/*
 * CHelper_GenericParallelEdgeCommTraversals.hpp
 *
 *  Created on: Jul 29, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CHELPER_GENERICPARALLELEDGECOMMTRAVERSALS_HPP_
#define CHELPER_GENERICPARALLELEDGECOMMTRAVERSALS_HPP_


/**
 * this class implements helper methods which abstract the different phases for
 * EDGE COMM TRAVERSALS
 *
 * in particular:
 *  a) first forward traversal
 *  b) edge communication
 *  c) last traversal
 */
template<
	typename CPartition_,
	typename CSimulation_Parallel_PartitionHandler,
	typename TEdgeCommSimulationSubClass,
	typename TEdgeData,
	typename CStackAccessors
>
class CGenericParallelEdgeCommTraversals
{
	typedef CPartition_ExchangeEdgeCommData<CPartition_, TEdgeData, CStackAccessors> CPartition_ExchangeEdgeCommData_;
	typedef typename CPartition_::CSimulationStacks_ CSimulationStacks_;

public:
	/**
	 * constructor
	 */
	CGenericParallelEdgeCommTraversals()
	{
	}


	/**
	 * run the edge comm traversals
	 */
	void action(
			TEdgeCommSimulationSubClass CSimulation_Parallel_PartitionHandler::*i_simulationSubClass,
			CPartition_ExchangeEdgeCommData_ CSimulation_Parallel_PartitionHandler::*i_simulationEdgeCommSubClass,
			CGenericTreeNode<CPartition_> *i_rootNode
	)
	{
		// first pass: setting all edges to type 'new' creates data to exchange with neighbors
		i_rootNode->traverse_PartitionTreeNode_Parallel(
				[=](CPartition_ *node)
				{
					(node->cSimulation_SubPartitionHandler->*(i_simulationSubClass)).actionFirstPass(node->cStacks);
				}
		);


		// exchange data
		i_rootNode->traverse_PartitionTreeNode_Parallel(
				[=](CPartition_ *node)
				{
					(node->cSimulation_SubPartitionHandler->*(i_simulationEdgeCommSubClass)).pullEdgeCommData();
				}
		);


		// second pass: setting all edges to type 'old' reads data from adjacent partitions
		i_rootNode->traverse_PartitionTreeNode_Parallel(
				[=](CPartition_ *node)
				{
					(node->cSimulation_SubPartitionHandler->*(i_simulationSubClass)).actionSecondPass_Parallel(node->cStacks);
				}
		);
	}
};

#endif /* CADAPTIVITYTRAVERSALS_HPP_ */
