/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *
 *  Created on: Jan 10, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */
#ifndef CTRAVERSATORS_CSTRUCTURE_ELEMENT_SETUP_PARALLEL_HPP_
#define CTRAVERSATORS_CSTRUCTURE_ELEMENT_SETUP_PARALLEL_HPP_

#include <string.h>
#include <stdint.h>
#include "libsierpi/stacks/CSimulationStacks.hpp"
#include "libsierpi/triangle/CTriangle_Factory.hpp"


namespace sierpi
{
namespace travs
{

template <typename p_CSimulationStacks>
class CSetup_Structure_ElementData
{
	typedef p_CSimulationStacks CSimulationStacks;
	typedef typename CSimulationStacks::TElementData TElementData;

public:
	CStack<char> *structure_stack;
	CStack<TElementData> *element_data_stack;
	TElementData *element_value;

	void setup(
			CSimulationStacks *io_simulationStacks,
			const char *i_structure_string,
			const TElementData *i_element_value,
			bool i_reversed = false
	)
	{
		structure_stack = &io_simulationStacks->structure_stacks.forward;
		element_data_stack = &io_simulationStacks->element_data_stacks.forward;

		element_value = i_element_value;


		if (!i_reversed)
		{
			for (const char *c = i_structure_string; *c != '\0'; c++)
			{
				if (*c == '0')
				{
					structure_stack->push(0);
					element_data_stack->push(*element_value);
				}
				else
				{
					structure_stack->push(1);
				}
			}
		}
		else
		{
			size_t length = strlen(i_structure_string);

			for (int i = length-1; i >= 0; i--)
			{
				if (i_structure_string[i] == '0')
				{
					structure_stack->push(0);
					element_data_stack->push(*element_value);
				}
				else
				{
					structure_stack->push(1);
				}
			}
		}

		assert(structure_stack->structure_isValidTriangle());
	}


	uint64_t setup(
			CSimulationStacks *io_simulationStacks,
			int p_depth,
			TElementData *p_element_value)
	{
		structure_stack = &io_simulationStacks->structure_stacks.forward;
		element_data_stack = &io_simulationStacks->element_data_stacks.forward;

		assert(p_depth >= 0);

		element_value = p_element_value;

		structure_stack->clear();
		element_data_stack->clear();

		assert(structure_stack->isEmpty());
		assert(element_data_stack->isEmpty());

		pushStackElement(p_depth);
		return (1<<(uint64_t)p_depth);
	}

private:
	void pushStackElement(int depth)
	{
		if (depth == 0)
		{
			structure_stack->push(0);
			element_data_stack->push_PtrValue(element_value);
			return;
		}

		depth--;
		pushStackElement(depth);
		pushStackElement(depth);

		// postfix push
		structure_stack->push(1);
	}
};

}
}

#endif /* CSTRUCTURE_SETUP_H_ */
