/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *
 *  Created on: Jan 10, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CTRAVERSATOR_VERTICES_PARALLEL_HPP_
#define CTRAVERSATOR_VERTICES_PARALLEL_HPP_

#include "libsierpi/triangle/CTriangle_Factory.hpp"
#include "libsierpi/stacks/CFBStacks.hpp"

namespace sierpi
{
namespace travs
{

template <class CKernelClass, typename CSimulationStacks>
class CTraversator_Vertices
{
public:
	/**
	 * implementation of kernel class
	 */
	CKernelClass kernelClass;

	/**
	 * a few typedefs which could be utilized by the traversator to get more informations
	 */
	typedef typename CKernelClass::TVertexScalar TVertexScalar;
	typedef CTraversator_Vertices TThisClass;

	CStackReaderTopDown<char> structure_lifo_in;

	/**
	 * include automagically generated code
	 */
#define SIERPI_PARALLEL_CODE	1
	#include "auto/CTraversator_Vertices_autogenerated.hpp"
#undef SIERPI_PARALLEL_CODE
#include "../CTraversatorClassInc_SetupSinglePass.hpp"



	void action(
			CSimulationStacks *cSimulationStacks
	)
	{
		structure_lifo_in.setup(cSimulationStacks->structure_stacks.forward);

		kernelClass.traversal_pre_hook();

		(this->*sfcRecursiveMethod_Forward)(
				triangleFactory.vertices[0][0], triangleFactory.vertices[0][1],
				triangleFactory.vertices[1][0], triangleFactory.vertices[1][1],
				triangleFactory.vertices[2][0], triangleFactory.vertices[2][1]
			);

		kernelClass.traversal_post_hook();
	}
};

}
}

#endif /* CSTRUCTURE_VERTEXDATA_H_ */
