/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 * CSimulationCommon.hpp
 *
 *  Created on: Jul 5, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#include <stdint.h>
#include "lib/CError.hpp"

#ifndef CSIMULATIONCOMMON_HPP_
#define CSIMULATIONCOMMON_HPP_

class CSimulationCommon
{
public:
	/**
	 * true, if the simulation should be driven
	 */
	bool run_simulation;

	/**
	 * true, if some error happened. Then no method should be executed
	 */
	CError error;


	/**
	 * timesteps which should be simulated per frame
	 */
	int timesteps_per_frame;

	/**
	 * initial recursion depth during setup
	 */
	int initial_recursion_depth;

	/**
	 * minimum and maximum recursion depth during refinements / coarsenings
	 */
	int min_relative_recursion_depth;
	int max_relative_recursion_depth;

	/**
	 * the time-step size
	 */
	float timestep_size;

	/**
	 * domain length of unit square
	 */
	TTsunamiDataScalar domain_length;

	/**
	 * overall number of triangles in domain
	 */
	uint64_t number_of_triangles;


	/**
	 * number of simulation handler partitions
	 */
	unsigned int number_of_simulation_handler_partitions;


	/**
	 * number of triangles when to split the partition
	 */
	uint64_t partition_split_workload_size;

	/**
	 * number of overall triangles of 2 partitions when to join the partition
	 */
	uint64_t partition_join_workload_size;


	/**
	 * set the number of triangles in a single partition when to split/join
	 */
public:
	void setSplitJoinSizes(
			uint64_t i_partition_split_workload_size,
			uint64_t i_partition_join_workload_size
		)
	{
		partition_split_workload_size = i_partition_split_workload_size;
		partition_join_workload_size = i_partition_join_workload_size;
	}


public:
	void setInitialRecursionDepth(int p_initial_recursion_depth)
	{
		initial_recursion_depth = p_initial_recursion_depth;
	}


	/**
	 * setup relative minimum recursion depth
	 */
	void setMinRelativeRecursionDepth(int p_min_recursion_depth)
	{
		assert(min_relative_recursion_depth <= 0);
		min_relative_recursion_depth = p_min_recursion_depth;
	}


	void setMaxRelativeRecursionDepth(int p_max_recursion_depth)
	{
		max_relative_recursion_depth = p_max_recursion_depth;
	}


	CSimulationCommon()	:
		run_simulation(false),
		initial_recursion_depth(6),
		min_relative_recursion_depth(0),
		max_relative_recursion_depth(6),
		timestep_size(-1),
		domain_length(100000.0),
		number_of_triangles(0),
		number_of_simulation_handler_partitions(0)
	{
	}


	virtual ~CSimulationCommon()
	{

	}
};

#endif /* CSIMULATIONCOMMON_H_ */
