/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 * CSimulationTsunami_Config.hpp
 *
 *  Created on: Sep 9, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CSIMULATIONTSUNAMI_CONFIG_HPP_
#define CSIMULATIONTSUNAMI_CONFIG_HPP_

class CSimulationTsunami_Config
{
public:
	/**
	 * thresholds for coarsening/refinement
	 */
	TTsunamiDataScalar coarsen_threshold;
	TTsunamiDataScalar refine_threshold;

	/**
	 * element data during setup
	 */
	CTsunamiElementData element_data_setup;

	/**
	 * Dirichlet boundary edge data
	 */
	CTsunamiEdgeData dirichlet_boundary_edge_data;

	/**
	 * element data to be modified by external event (e. g. user input)
	 */
	CTsunamiElementData element_data_modifier;

	/**
	 * number of triangles when to split the partition
	 */
	uint64_t partition_split_workload_size;

	/**
	 * number of overall triangles of 2 partitions when to join the partition
	 */
	uint64_t partition_join_workload_size;

	/**
	 * world scene id to setup
	 */
	int worldSceneId;

	/**
	 * if this variable is set to true, the timestep size is manually specified
	 */
	bool timestepSize_override;


	inline void outputVerboseInformation()
	{
		std::cout << " + Flux solver: " << SIMULATION_TSUNAMI_FLUX_SOLVER_STRING << std::endl;
		std::cout << " + Order of basis functions: " << SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS << std::endl;
	}



	CSimulationTsunami_Config()
	{
		worldSceneId = 3;

		TTsunamiDataScalar baseHeight = 50;
		TTsunamiDataScalar columnHeight = 51;

#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
		element_data_modifier.dofs.h = columnHeight;
		element_data_modifier.dofs.qx = 0;
		element_data_modifier.dofs.qy = 0;
		element_data_modifier.dofs.b = -baseHeight;
#else
		element_data_modifier.hyp_edge.h = columnHeight;
		element_data_modifier.hyp_edge.qx = 0;
		element_data_modifier.hyp_edge.qy = 0;
		element_data_modifier.hyp_edge.b = -baseHeight;

		element_data_modifier.left_edge.h = columnHeight;
		element_data_modifier.left_edge.qx = 0;
		element_data_modifier.left_edge.qy = 0;
		element_data_modifier.left_edge.b = -baseHeight;

		element_data_modifier.right_edge.h = columnHeight;
		element_data_modifier.right_edge.qx = 0;
		element_data_modifier.right_edge.qy = 0;
		element_data_modifier.right_edge.b = -baseHeight;
#endif

		element_data_modifier.refine = false;
		element_data_modifier.coarsen = false;



#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS == 0

		element_data_setup.dofs.h = baseHeight;
		element_data_setup.dofs.qx = 0;
		element_data_setup.dofs.qy = 0;
		element_data_setup.dofs.b = -baseHeight;

		dirichlet_boundary_edge_data.h = baseHeight;
		dirichlet_boundary_edge_data.qx = 0;
		dirichlet_boundary_edge_data.qy = 0;
		dirichlet_boundary_edge_data.b = -baseHeight;

#else

		element_data_setup.hyp_edge.h = baseHeight;
		element_data_setup.hyp_edge.qx = 0;
		element_data_setup.hyp_edge.qy = 0;
		element_data_setup.hyp_edge.b = -baseHeight;

		element_data_setup.left_edge.h = baseHeight;
		element_data_setup.left_edge.qx = 0;
		element_data_setup.left_edge.qy = 0;
		element_data_setup.left_edge.b = -baseHeight;

		element_data_setup.right_edge.h = baseHeight;
		element_data_setup.right_edge.qx = 0;
		element_data_setup.right_edge.qy = 0;
		element_data_setup.right_edge.b = -baseHeight;

		dirichlet_boundary_edge_data.h = baseHeight;
		dirichlet_boundary_edge_data.qx = 0;
		dirichlet_boundary_edge_data.qy = 0;
		dirichlet_boundary_edge_data.b = -baseHeight;
#endif

		element_data_setup.refine = false;
		element_data_setup.coarsen = false;


#if 0

		coarsen_threshold = 50.01;
		refine_threshold = 50.2;

#else

		// adaptive stuff
#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS == 0
		/*
		 * threshold for 0th order is given by surface height
		 */
		coarsen_threshold = 50.01;
		refine_threshold = 50.2;

#else
		/*
		 * threshold for 1st order is given by surface slope
		 */
		TTsunamiDataScalar unit_slope = CMath::abs((element_data_modifier.hyp_edge.h-dirichlet_boundary_edge_data.h)/element_data_modifier.hyp_edge.h);
		coarsen_threshold	= 0.002*unit_slope;
		refine_threshold	= 0.003*unit_slope;
#endif
#endif
		timestepSize_override = false;
	}
};

#endif /* CSIMULATIONTSUNAMI_CONFIG_HPP_ */
