/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 * CAdaptive_Tsunami_1stOrder_CoarsenRefine.hpp
 *
 *  Created on: Jul 1, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CADAPTIVE_TSUNAMI_0THORDER_COARSENREFINE_HPP_
#define CADAPTIVE_TSUNAMI_0THORDER_COARSENREFINE_HPP_

#include "config.h"
#include "../../types/CTsunamiTypes.hpp"

namespace sierpi
{
namespace kernels
{


class CAdaptive_Tsunami_0thOrder_CoarsenRefine
{

public:
	inline void refine_l_r(
			TTsunamiDataScalar normal_hypx,		TTsunamiDataScalar normal_hypy,
			TTsunamiDataScalar normal_rightx,	TTsunamiDataScalar normal_righty,
			TTsunamiDataScalar normal_leftx,	TTsunamiDataScalar normal_lefty,
			int depth,
			CTsunamiElementData *element,
			CTsunamiElementData *left_element,
			CTsunamiElementData *right_element
	)
	{
		*left_element = *element;
		*right_element = *element;


#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
		element->validation.setupRefineLeftAndRight(
				normal_hypx,	normal_hypy,
				normal_rightx,	normal_righty,
				normal_leftx,	normal_lefty,
				depth,
				&left_element->validation,
				&right_element->validation
			);
#endif
	}



	inline void refine_ll_r(
			TTsunamiDataScalar normal_hypx, TTsunamiDataScalar normal_hypy,
			TTsunamiDataScalar normal_rightx, TTsunamiDataScalar normal_righty,
			TTsunamiDataScalar normal_leftx, TTsunamiDataScalar normal_lefty,
			int depth,
			CTsunamiElementData *element,
			CTsunamiElementData *left_left_element,
			CTsunamiElementData *left_right_element,
			CTsunamiElementData *right_element
	)
	{
		// LEFT + RIGHT
		refine_l_r(	normal_hypx, normal_hypy,
					normal_rightx, normal_righty,
					normal_leftx, normal_lefty,
					depth,
					element, left_right_element, right_element
				);

		// LEFT childs (LEFT + RIGHT)
		refine_l_r(	normal_leftx, normal_lefty,
					normal_hypx, normal_hypy,
					-normal_hypy, normal_hypx,
					depth+1,
					left_right_element, left_left_element, left_right_element
				);
	}



	inline void refine_l_rr(
			TTsunamiDataScalar normal_hypx, TTsunamiDataScalar normal_hypy,
			TTsunamiDataScalar normal_rightx, TTsunamiDataScalar normal_righty,
			TTsunamiDataScalar normal_leftx, TTsunamiDataScalar normal_lefty,
			int depth,
			CTsunamiElementData *element,
			CTsunamiElementData *left_element,
			CTsunamiElementData *right_left_element,
			CTsunamiElementData *right_right_element
	)
	{
		// LEFT + RIGHT
		refine_l_r(	normal_hypx, normal_hypy,
					normal_rightx, normal_righty,
					normal_leftx, normal_lefty,
					depth,
					element, left_element, right_right_element);

		// RIGHT childs (LEFT + RIGHT)
		refine_l_r(	normal_rightx, normal_righty,
					normal_hypy, -normal_hypx,
					normal_hypx, normal_hypy,
					depth+1,
					right_right_element, right_left_element, right_right_element);
	}



	inline void refine_ll_rr(
			TTsunamiDataScalar normal_hypx, TTsunamiDataScalar normal_hypy,
			TTsunamiDataScalar normal_rightx, TTsunamiDataScalar normal_righty,
			TTsunamiDataScalar normal_leftx, TTsunamiDataScalar normal_lefty,
			int depth,
			CTsunamiElementData *element,
			CTsunamiElementData *left_left_element,
			CTsunamiElementData *left_right_element,
			CTsunamiElementData *right_left_element,
			CTsunamiElementData *right_right_element
	)
	{
		// RIGHT + LEFT
		refine_l_r(	normal_hypx, normal_hypy,
					normal_rightx, normal_righty,
					normal_leftx, normal_lefty,
					depth,
					element, left_right_element, right_right_element);

		// LEFT childs (LEFT + RIGHT)
		refine_l_r(	normal_leftx, normal_lefty,
					normal_hypx, normal_hypy,
					-normal_hypy, normal_hypx,
					depth+1,
					left_right_element, left_left_element, left_right_element);

		// LEFT childs (LEFT + RIGHT)
		refine_l_r(	normal_rightx, normal_righty,
					normal_hypy, -normal_hypx,
					normal_hypx, normal_hypy,
					depth+1,
					right_right_element, right_left_element, right_right_element);
	}


	inline void coarsen(
			TTsunamiDataScalar normal_hypx, TTsunamiDataScalar normal_hypy,
			TTsunamiDataScalar normal_rightx, TTsunamiDataScalar normal_righty,
			TTsunamiDataScalar normal_leftx, TTsunamiDataScalar normal_lefty,

			int depth,

			CTsunamiElementData *left_element,
			CTsunamiElementData *right_element,
			CTsunamiElementData *coarsed_element
	)
	{
		coarsed_element->dofs.h = (left_element->dofs.h + right_element->dofs.h)*(TTsunamiDataScalar)0.5;
		coarsed_element->dofs.qx = (left_element->dofs.qx + right_element->dofs.qx)*(TTsunamiDataScalar)0.5;
		coarsed_element->dofs.qy = (left_element->dofs.qy + right_element->dofs.qy)*(TTsunamiDataScalar)0.5;
		coarsed_element->dofs.b = (left_element->dofs.b + right_element->dofs.b)*(TTsunamiDataScalar)0.5;
		coarsed_element->refine = false;
		coarsed_element->coarsen = false;

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
		coarsed_element->validation.setupCoarsen(
				normal_hypx, normal_hypy,
				normal_rightx, normal_righty,
				normal_leftx, normal_lefty,
				depth,
				&left_element->validation,
				&right_element->validation
		);
#endif
	}

};


}
}

#endif /* CADAPTIVE_TSUNAMI_1STORDER_COARSENREFINE_HPP_ */
