/*
 * Copyright (C) 2011 Technische Universitaet Muenchen
 * This file is part of the Sierpi project. For conditions of distribution and
 * use, please see the copyright notice in the file 'copyright.txt' at the root
 * directory of this package and the copyright notice at http://www5.in.tum.de/sierpi
 *
 *  Created on: Feb 23, 2011
 *      Author: Martin Schreiber (martin.schreiber@in.tum.de)
 */

#ifndef CTSUNAMI_TYPES_HPP
#define CTSUNAMI_TYPES_HPP

#include "libsierpi/stacks/CSimulationStacks.hpp"
#include "libmath/CMath.hpp"
#include "config.h"
#include "../CTsunamiConfig.hpp"


#if !DEBUG
#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
#error "COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION activated in non-debug mode!"
#endif
#endif

#if COMPILE_WITH_DEBUG_TSUNAMI_ELEMENTDATA_VALIDATION
#include <string.h>
#endif

#if SIMULATION_TSUNAMI_ORDER_OF_BASIS_FUNCTIONS==0
	#include "CTsunamiTypes_0thOrder.hpp"
#else
	#include "CTsunamiTypes_1stOrder.hpp"
#endif

class CTsunamiSimulationTypes
{
	typedef CTsunamiElementData CElementData;
	typedef CTsunamiEdgeData CEdgeData;

	typedef TTsunamiDataScalar TDataScalar;
	typedef TTsunamiVertexScalar TVertexScalar;
};



typedef CSimulationStacks<CTsunamiElementData, CTsunamiEdgeData> CTsuanmiSimulationStacks;


/*
inline void getUnitCatAndHypLengthForDepth(
		int i_depth,
		TTsunamiDataScalar *o_catLength,
		TTsunamiDataScalar *hypLength
)
{
	int depth_even = i_depth >> 1;

	o_catLength = (TTsunamiDataScalar)1.0/(TTsunamiDataScalar)(1<<depth_even);

	if (i_depth & 1)
		o_catLength = (TTsunamiDataScalar)0.5*CMath::sqrt2<TTsunamiDataScalar>()*o_catLength;




	i_depth += 1;
	int depth_even = i_depth >> 1;

	TTsunamiDataScalar hypotenuse = (TTsunamiDataScalar)1.0/(TTsunamiDataScalar)(1<<depth_even);

	if (i_depth & 1)
		hypotenuse = (TTsunamiDataScalar)0.5*CMath::sqrt2<TTsunamiDataScalar>()*hypotenuse;

	return hypotenuse*2.0;
}*/



inline TTsunamiDataScalar getUnitCathetusLengthForDepth(int i_depth)
{
#if 1
	/*
	 * TODO: increase precision!!!
	 */
	static TTsunamiDataScalar catheti_lengths[64] = {
		1,
		0.707107,
		0.5,
		0.353553,
		0.25,
		0.176777,
		0.125,
		0.0883883,
		0.0625,
		0.0441942,
		0.03125,
		0.0220971,
		0.015625,
		0.0110485,
		0.0078125,
		0.00552427,
		0.00390625,
		0.00276214,
		0.00195312,
		0.00138107,
		0.000976562,
		0.000690534,
		0.000488281,
		0.000345267,
		0.000244141,
		0.000172633,
		0.00012207,
		8.63167e-05,
		6.10352e-05,
		4.31584e-05,
		3.05176e-05,
		2.15792e-05,
		1.52588e-05,
		1.07896e-05,
		7.62939e-06,
		5.3948e-06,
		3.8147e-06,
		2.6974e-06,
		1.90735e-06,
		1.3487e-06,
		9.53674e-07,
		6.7435e-07,
		4.76837e-07,
		3.37175e-07,
		2.38419e-07,
		1.68587e-07,
		1.19209e-07,
		8.42937e-08,
		5.96046e-08,
		4.21468e-08,
		2.98023e-08,
		2.10734e-08,
		1.49012e-08,
		1.05367e-08,
		7.45058e-09,
		5.26836e-09,
		3.72529e-09,
		2.63418e-09,
		1.86265e-09,
		1.31709e-09,
		9.31323e-10,
		6.58545e-10,
		4.65661e-10,
		3.29272e-10,
	};

	return catheti_lengths[i_depth];

#else

	TTsunamiDataScalar cathetus = (TTsunamiDataScalar)1.0/(TTsunamiDataScalar)(1<<depth_even);

	if (i_depth & 1)
		cathetus = (TTsunamiDataScalar)0.5*CMath::sqrt2<TTsunamiDataScalar>()*cathetus;

	return cathetus;
#endif
}


inline TTsunamiDataScalar getUnitHypotenuseLengthForDepth(int i_depth)
{
#if 1
	static TTsunamiDataScalar hypotenuse_lengths[64] = {
		1.41421,
		1,
		0.707107,
		0.5,
		0.353553,
		0.25,
		0.176777,
		0.125,
		0.0883883,
		0.0625,
		0.0441942,
		0.03125,
		0.0220971,
		0.015625,
		0.0110485,
		0.0078125,
		0.00552427,
		0.00390625,
		0.00276214,
		0.00195312,
		0.00138107,
		0.000976562,
		0.000690534,
		0.000488281,
		0.000345267,
		0.000244141,
		0.000172633,
		0.00012207,
		8.63167e-05,
		6.10352e-05,
		4.31584e-05,
		3.05176e-05,
		2.15792e-05,
		1.52588e-05,
		1.07896e-05,
		7.62939e-06,
		5.3948e-06,
		3.8147e-06,
		2.6974e-06,
		1.90735e-06,
		1.3487e-06,
		9.53674e-07,
		6.7435e-07,
		4.76837e-07,
		3.37175e-07,
		2.38419e-07,
		1.68587e-07,
		1.19209e-07,
		8.42937e-08,
		5.96046e-08,
		4.21468e-08,
		2.98023e-08,
		2.10734e-08,
		1.49012e-08,
		1.05367e-08,
		7.45058e-09,
		5.26836e-09,
		3.72529e-09,
		2.63418e-09,
		1.86265e-09,
		1.31709e-09,
		9.31323e-10,
		6.58545e-10,
		4.65661e-10,
	};

	return hypotenuse_lengths[i_depth];

#else

	i_depth += 1;
	int depth_even = i_depth >> 1;

	TTsunamiDataScalar hypotenuse = (TTsunamiDataScalar)1.0/(TTsunamiDataScalar)(1<<depth_even);

	if (i_depth & 1)
		hypotenuse = (TTsunamiDataScalar)0.5*CMath::sqrt2<TTsunamiDataScalar>()*hypotenuse;

	return hypotenuse*2.0;

#endif
}
#endif
